@@ -5,7 +5,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.15';
+our $VERSION = '0.16';
 $VERSION =~ tr/_//d;
 
 BEGIN {
@@ -13,17 +13,21 @@ BEGIN {
   $Carp::CarpInternal{ +__PACKAGE__ }++;
 }
 
+use constant CHOMP_DOT => $Carp::VERSION < 1.25;
+
 sub _warn { warn &_longmess }
 
 sub _die { die ref $_[0] ? @_ : &_longmess }
 
 sub _longmess {
-  if ($_[-1] =~ /\n\z/) {
+  if (CHOMP_DOT && $_[-1] =~ /\.\n\z/) {
     my $arg = pop @_;
-    $arg =~ s/(.*) at [^\n]*? line [0-9]+\.?\n$/$1/s;
+    $arg =~ s/\.\n\z/\n/;
     push @_, $arg;
   }
-  &Carp::longmess;
+  my $mess = &Carp::longmess;
+  $mess =~ s/( at .*?\n)\1/$1/s;    # Suppress duplicate tracebacks
+  $mess;
 }
 
 my @HOOKS = qw(__DIE__ __WARN__);
@@ -32,7 +36,7 @@ my %OLD_SIG;
 sub import {
   my $class = shift;
   return if $OLD_SIG{$class};
-  @{ $OLD_SIG{$class} }{@HOOKS} = @SIG{@HOOKS};
+  @{ $OLD_SIG{$class} }{ @HOOKS, 'Verbose' } = (@SIG{@HOOKS}, $Carp::Verbose);
 
   @SIG{@HOOKS} = ($class->can('_die'), $class->can('_warn'));
   $Carp::Verbose = 'verbose';    # makes carp() cluck and croak() confess
@@ -41,8 +45,8 @@ sub import {
 sub unimport {
   my $class = shift;
   return unless $OLD_SIG{$class};
-  @SIG{@HOOKS} = @{ delete $OLD_SIG{$class} }{@HOOKS};
-  undef $Carp::Verbose;
+  no if "$]" <= 5.008008, 'warnings' => 'uninitialized';
+  (@SIG{@HOOKS}, $Carp::Verbose) = @{ delete $OLD_SIG{$class} }{ @HOOKS, 'Verbose' };
 }
 
 1;
@@ -171,7 +175,7 @@ Adriano Ferreira, E<lt>ferreira@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005-2013 by Adriano R. Ferreira
+Copyright (C) 2005-2013, 2018 by Adriano Ferreira
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
